REM Sub List.Drives for Whatis.

REM $DYNAMIC
DEFINT A-Z

DECLARE SUB FreeSpace (V$)
DECLARE SUB TotalSpace (V$)
DECLARE SUB Check.Drive3 (V%, T#)
DECLARE SUB Suffix2 (V#, V$)

DECLARE SUB Vlabel (c$, c%)
DECLARE SUB Vserial (c$, c%)

DIM SHARED Out3 AS STRING

DIM SHARED BASIC.DTA.SEG AS INTEGER
DIM SHARED BASIC.DTA.OFF AS INTEGER

' define color values
CONST Black = 0
CONST Blue = 1
CONST Plain = 7
CONST Green = 10
CONST Cyan = 11
CONST Red = 12
CONST Magenta = 13
CONST Yellow = 14
CONST White = 15

' constant declarations
CONST FalseD = 0#
CONST TrueD = -1#
CONST False = 0
CONST True = -1
CONST Nul = ""

TYPE RegTypeX
 AX AS INTEGER
 BX AS INTEGER
 CX AS INTEGER
 DX AS INTEGER
 BP AS INTEGER
 SI AS INTEGER
 DI AS INTEGER
 Flags AS INTEGER
 DS AS INTEGER
 ES AS INTEGER
END TYPE

' declare interrupt functions
DIM SHARED InregsX AS RegTypeX, OutregsX AS RegTypeX
DECLARE SUB InterruptX (N AS INTEGER, I AS RegTypeX, O AS RegTypeX)

' Bios Parameter Block Structure
TYPE BPBtype
 Level AS STRING * 2
 Serial AS STRING * 4
 Label AS STRING * 11
 System AS STRING * 8
END TYPE

' DOS Data Transfer Area
TYPE DTAtype
 Drive AS STRING * 1
 SearchTemplate AS STRING * 11
 SearchAttr AS STRING * 1
 EntryCount AS STRING * 2
 ClusterNumber AS STRING * 2
 Reserved AS STRING * 4
 FileAttr AS STRING * 1
 Filetime AS STRING * 2
 Filedate AS STRING * 2
 FileSize AS STRING * 4
 ASCIIZfilename AS STRING * 13
END TYPE

' check drive exists
SUB Check.Drive3 (V, T#)
 ON LOCAL ERROR GOTO Disk.Error
 ' check disk space.
 InregsX.AX = &H3600
 InregsX.DX = V
 CALL InterruptX(&H21, InregsX, OutregsX)
 IF OutregsX.AX = &HFFFF THEN
    T# = 0#
 ELSE
    T# = -1#
 END IF
 EXIT SUB
Disk.Error:
 T# = 0#
 EXIT SUB
END SUB

' get drive freespace
SUB FreeSpace (Var$)
 ' check disk space.
 InregsX.AX = &H3600
 InregsX.DX = ASC(UCASE$(Var$)) - 64
 CALL InterruptX(&H21, InregsX, OutregsX)
 IF OutregsX.AX = &HFFFF THEN
    T# = 0#
 ELSE
'   free space on drive in bytes is AX * BX * CX
    T1# = OutregsX.AX: IF T1# < 0# THEN T1# = T1# + 65536#
    T2# = OutregsX.BX: IF T2# < 0# THEN T2# = T2# + 65536#
    T3# = OutregsX.CX: IF T3# < 0# THEN T3# = T3# + 65536#
    T# = T1# * T2# * T3#
 END IF
 Var$ = LTRIM$(STR$(T#))
END SUB

' lists specified drives
SUB List.Drives (VarQ)
 GOSUB Drive.Header
 l = 0
 FOR c% = 1 TO 26
    GOSUB DisplayDrive
    IF h = 20 THEN
       h = 0
       CALL More.Prompt("-more-", "yn" + CHR$(13), Output.Char$)
       IF LCASE$(Output.Char$) = "n" THEN
          EXIT FOR
       END IF
       GOSUB Drive.Header
    END IF
 NEXT
 IF q = 0 THEN
    PRINT "<none>"
 ELSE
    COLOR White, Black
    PRINT "Total drives"; l
 END IF
 EXIT SUB

DisplayDrive:
 CALL Check.Drive3(c, T#)
 IF T# THEN
    h = h + 1
    l = l + 1
    q = -1
    ' get drive
    c$ = CHR$(c% + 64)
    d$ = c$: e$ = c$: f$ = c$: g$ = c$: h$ = c$
    COLOR White, Black
    PRINT c$; ":    ";
    COLOR Yellow, Black
    ' get volume label
    CALL Vlabel(g$, c%)
    z$ = g$
    z$ = z$ + SPACE$(13 - LEN(z$))
    PRINT z$;
    COLOR Green, Black
    ' get volume serial
    CALL Vserial(h$, c%)
    Out3$ = h$
    Out3$ = LEFT$(Out3$, 9)
    IF Out3$ = "" THEN Out3$ = "<unknown>"
    Out3$ = Out3$ + SPACE$(9 - LEN(Out3$))
    PRINT Out3$;
    COLOR Cyan, Black
    Out3$ = c$
    CALL TotalSpace(d$)
    x# = INT(VAL(d$))
    x1# = x#
    IF x# > 0# THEN
       CALL Suffix2(x#, S$) ' 1,024.0 KB
       S$ = LEFT$(S$, 11)
       PRINT SPACE$(11 - LEN(S$)) + S$;
    ELSE
       PRINT "      <n/a>";
    END IF
    Out3$ = c$
    CALL FreeSpace(e$)
    y# = INT(VAL(e$))
    y1# = y#
    IF y# > 0# THEN
       CALL Suffix2(y#, S$) ' 1,024.0 KB
       S$ = LEFT$(S$, 11)
       PRINT SPACE$(11 - LEN(S$)) + S$;
    ELSE
       PRINT "      <n/a>";
    END IF
    IF x1# > 0# OR y1# > 0# THEN
       z# = x1# - y1#
       CALL Suffix2(z#, S$) ' 1,024.0 KB
       S$ = LEFT$(S$, 11)
       PRINT SPACE$(11 - LEN(S$)) + S$
    ELSE
       PRINT "      <n/a>"
    END IF
 END IF
 RETURN

Drive.Header:
 h = 2
 COLOR White, Black
 PRINT "Drive Label        Serial         Total       Free       Used"
 PRINT "-------------------------------------------------------------"
 RETURN
END SUB

' calculate byte suffix
SUB Suffix2 (Var#, Var3$)

 REM B  (Byte) = 00x - 0FFx (hexidecimal zero-based)
 REM KB (Kilobyte) = 1024 B
 REM MB (Megabyte) = 1024 KB (1 MB B)
 REM GB (Gigabyte) = 1024 MB
 REM TB (Terabyte) = 1024 GB (1 MB MB)
 REM PB (Petabyte) = 1024 TB
 REM EB (Exabyte)  = 1024 PB (1 MB TB)
 REM ZB (Zettabyte) = 1024 EB
 REM YB (Yottabyte) = 1024 ZB (1 MB EB)

 ' store double
 VarX# = Var#

 ' -1.#IND and 1.#INF
 IF INSTR(STR$(VarX#), "#") THEN
    Var3$ = LTRIM$(STR$(VarX#))
    EXIT SUB
 END IF

 ' get sign
 IF VarX# < 0# THEN
    Sign = -1
    VarX# = ABS(VarX#)
 END IF

 ' calculate bytes
 TempA = False
 DO
    IF VarX# >= 1024 THEN
       VarX# = VarX# / 1024
       TempA = TempA + 1
       IF TempA = 8 THEN
	  EXIT DO
       END IF
    ELSE
       EXIT DO
    END IF
 LOOP

 ' calculate byte string
 Var3$ = FORMAT$(VarX#, "#,##0.0;;")

 ' calculate byte suffix
 Var$ = Nul
 IF TempA > 0 THEN
    Var$ = MID$("KMGTPEZY", TempA, 1)
 END IF
 Var3$ = Var3$ + " " + Var$ + "B"

 ' calculate byte sign
 IF Sign THEN
    Var3$ = "-" + Var3$
 END IF
END SUB

' get drive totalspace
SUB TotalSpace (Var$)
 ' check disk space.
 InregsX.AX = &H3600
 InregsX.DX = ASC(UCASE$(Var$)) - 64
 CALL InterruptX(&H21, InregsX, OutregsX)
 IF OutregsX.AX = &HFFFF THEN
    T# = 0#
 ELSE
'   total space on drive in bytes is AX * CX * DX
    T1# = OutregsX.AX: IF T1# < 0# THEN T1# = T1# + 65536#
    T2# = OutregsX.CX: IF T2# < 0# THEN T2# = T2# + 65536#
    T3# = OutregsX.DX: IF T3# < 0# THEN T3# = T3# + 65536#
    T# = T1# * T2# * T3#
 END IF
 Var$ = LTRIM$(STR$(T#))
END SUB

SUB Vlabel (c$, V%)
 DIM ASCIIZ AS STRING * 260
 DIM ParseDTA AS DTAtype

 ' store basic dta.
 InregsX.AX = &H2F00
 CALL InterruptX(&H21, InregsX, OutregsX)
 BASIC.DTA.SEG = OutregsX.ES
 BASIC.DTA.OFF = OutregsX.BX

 c$ = ""
 IF V% >= 1 AND V% <= 26 THEN
    CALL Check.Drive3(V%, Temp#)
 END IF
 IF Temp# = 0# THEN
    EXIT SUB
 END IF
 ASCIIZ = CHR$(V% + 64) + ":\*.*" + CHR$(0)
 ' restore directory search dta
 InregsX.AX = &H1A00
 InregsX.DS = VARSEG(ParseDTA)
 InregsX.DX = VARPTR(ParseDTA)
 CALL InterruptX(&H21, InregsX, OutregsX)
 ' find volume label
 InregsX.AX = &H4E00
 InregsX.CX = &H8
 InregsX.DS = VARSEG(ASCIIZ)
 InregsX.DX = VARPTR(ASCIIZ)
 CALL InterruptX(&H21, InregsX, OutregsX)
 ' check findfirst error
 IF (OutregsX.Flags AND &H1) = &H0 THEN
    c$ = ParseDTA.ASCIIZfilename
    IF INSTR(c$, CHR$(0)) THEN
       c$ = LEFT$(c$, INSTR(c$, CHR$(0)) - 1)
    END IF
 END IF
 ' restore basic dta
 InregsX.AX = &H1A00
 InregsX.DS = BASIC.DTA.SEG
 InregsX.DX = BASIC.DTA.OFF
 CALL InterruptX(&H21, InregsX, OutregsX)
END SUB

SUB Vserial (c$, V%)
DIM BPBfile AS BPBtype
 c$ = ""
 IF V >= 1 AND V <= 26 THEN
    CALL Check.Drive3(V, Temp#)
 END IF
 IF Temp# = 0# THEN
    EXIT SUB
 END IF
 ' get volume info
 InregsX.AX = &H6900
 InregsX.BX = V%
 InregsX.DS = VARSEG(BPBfile)
 InregsX.DX = VARPTR(BPBfile)
 CALL InterruptX(&H21, InregsX, OutregsX)
 ' check flag error
 IF (OutregsX.Flags AND &H1) = &H0 THEN
    ' store volume serial number
    FOR Serial.Digit = 4 TO 1 STEP -1
       IF Serial.Digit = 2 THEN
	  c$ = c$ + "-"
       END IF
       Serial.Value = ASC(MID$(BPBfile.Serial, Serial.Digit, 1))
       c$ = c$ + RIGHT$(HEX$(Serial.Value + &H100), 2)
    NEXT
 END IF
END SUB

